@@ -22,7 +22,7 @@ from typing import List, Tuple
 import jinja2
 import torch
 
-from .core import load_cuda_ops, sm90a_nvcc_flags
+from .core import logger, load_cuda_ops, sm90a_nvcc_flags
 from .env import FLASHINFER_CSRC_DIR, FLASHINFER_GEN_SRC_DIR
 from .utils import (
     dtype_map,
@@ -101,7 +101,8 @@ def get_single_decode_uri(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -110,7 +111,8 @@ def get_single_decode_uri(
         f"single_decode_with_kv_cache_dtype_q_{filename_safe_dtype_map[dtype_q]}_"
         f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
         f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
-        f"head_dim_{head_dim}_"
+        f"head_dim_qk_{head_dim_qk}_"
+        f"head_dim_vo_{head_dim_vo}_"
         f"posenc_{pos_encoding_mode}_"
         f"use_swa_{use_sliding_window}_"
         f"use_logits_cap_{use_logits_soft_cap}"
@@ -122,7 +124,8 @@ def get_batch_decode_uri(
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
     dtype_idx: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -132,30 +135,101 @@ def get_batch_decode_uri(
         f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
         f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
         f"dtype_idx_{filename_safe_dtype_map[dtype_idx]}_"
-        f"head_dim_{head_dim}_"
+        f"head_dim_qk_{head_dim_qk}_"
+        f"head_dim_vo_{head_dim_vo}_"
         f"posenc_{pos_encoding_mode}_"
         f"use_swa_{use_sliding_window}_"
         f"use_logits_cap_{use_logits_soft_cap}"
     )
 
 
+def get_batch_mla_uri(
+    dtype_q: torch.dtype,
+    dtype_kv: torch.dtype,
+    dtype_o: torch.dtype,
+    dtype_idx: torch.dtype,
+    head_dim_ckv: int,
+    head_dim_kpe: int,
+) -> str:
+    return (
+        f"batch_mla_attention_dtype_q_{filename_safe_dtype_map[dtype_q]}_"
+        f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
+        f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
+        f"dtype_idx_{filename_safe_dtype_map[dtype_idx]}_"
+        f"head_dim_ckv_{head_dim_ckv}_"
+        f"head_dim_kpe_{head_dim_kpe}"
+    )
+
+
+def gen_batch_mla_module(
+    dtype_q: torch.dtype,
+    dtype_kv: torch.dtype,
+    dtype_o: torch.dtype,
+    dtype_idx: torch.dtype,
+    head_dim_ckv: int,
+    head_dim_kpe: int,
+):
+    uri = get_batch_mla_uri(
+        dtype_q,
+        dtype_kv,
+        dtype_o,
+        dtype_idx,
+        head_dim_ckv,
+        head_dim_kpe,
+    )
+    gen_directory = FLASHINFER_GEN_SRC_DIR / uri
+    os.makedirs(gen_directory, exist_ok=True)
+
+    with open(FLASHINFER_CSRC_DIR / "batch_mla_config.jinja") as f:
+        config_templ = jinja2.Template(f.read())
+    generated_config_path = gen_directory / "batch_mla_config.inc"
+    write_if_different(
+        generated_config_path,
+        config_templ.render(
+            dtype_q=dtype_map[dtype_q],
+            dtype_kv=dtype_map[dtype_kv],
+            dtype_o=dtype_map[dtype_o],
+            dtype_idx=dtype_map[dtype_idx],
+            head_dim_ckv=head_dim_ckv,
+            head_dim_kpe=head_dim_kpe,
+        ),
+    )
+
+    source_paths = []
+    for filename in [
+        "batch_mla_plan.cu",
+        "batch_mla_run.cu",
+        "batch_mla_pybind.cu",
+    ]:
+        src_path = FLASHINFER_CSRC_DIR / filename
+        dest_path = gen_directory / filename
+        source_paths.append(dest_path)
+        with open(src_path, "r") as f:
+            source = f.read()
+        write_if_different(dest_path, source)
+
+    return load_cuda_ops(uri, source_paths)
+
+
 def get_batch_decode_mla_uri(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
     dtype_idx: torch.dtype,
-    head_dim: int,
+    head_dim_ckv: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
+    arc: str,
 ) -> str:
     return (
         f"batch_decode_mla_with_kv_cache_dtype_q_{filename_safe_dtype_map[dtype_q]}_"
         f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
         f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
         f"dtype_idx_{filename_safe_dtype_map[dtype_idx]}_"
-        f"head_dim_{head_dim}_"
+        f"head_dim_ckv{head_dim_ckv}_"
         f"use_swa_{use_sliding_window}_"
-        f"use_logits_cap_{use_logits_soft_cap}"
+        f"use_logits_cap_{use_logits_soft_cap}_"
+        f"arc_{arc}"
     )
 
 
@@ -165,9 +239,30 @@ def gen_batch_decode_mla_module(
     dtype_o: torch.dtype,
     dtype_idx: torch.dtype,
     head_dim: int,
+    num_qo_heads: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
+    use_tensor_cores: bool, 
 ):
+    cuda_arch_major = torch.cuda.get_device_properties(0).major
+    
+    if cuda_arch_major >= 9: # smem size of SM90 can accommodate all 128 qo-heads data
+        qo_tile_len = 128 
+    else:
+        qo_tile_len = 64
+    
+    if (
+            use_tensor_cores and
+            cuda_arch_major >= 8 and num_qo_heads % qo_tile_len == 0 and 
+            dtype_q == torch.float16 and dtype_kv == torch.float16 and 
+            dtype_o == torch.float16
+       ):
+        logger.info(f"Use tensor-core SM80 version of MLA decode kernel.")
+        arc = "sm80"
+    else: 
+        logger.info(f"Fall back to cuda-core version of MLA decode kernel.")
+        arc = "cuda_core"
+    
     uri = get_batch_decode_mla_uri(
         dtype_q,
         dtype_kv,
@@ -176,6 +271,7 @@ def gen_batch_decode_mla_module(
         head_dim,
         use_sliding_window,
         use_logits_soft_cap,
+        arc,
     )
     gen_directory = FLASHINFER_GEN_SRC_DIR / uri
     os.makedirs(gen_directory, exist_ok=True)
@@ -192,17 +288,27 @@ def gen_batch_decode_mla_module(
             dtype_idx=dtype_map[dtype_idx],
             head_dim_ckv=head_dim,
             head_dim_kpe=head_dim // 8,
+            qo_tile_len=qo_tile_len,
             use_sliding_window=str(use_sliding_window).lower(),
             use_logits_soft_cap=str(use_logits_soft_cap).lower(),
         ),
     )
+    
+    filenames = []
+    if arc == "sm80":
+        filenames = [
+                        "batch_decode_mla_cute_sm80.cu",
+                        "batch_decode_mla_pybind.cu",
+                    ]
+    else: 
+        filenames = [
+                        "batch_decode_mla_plan.cu",
+                        "batch_decode_mla_run.cu",
+                        "batch_decode_mla_pybind.cu",
+                    ]
 
     source_paths = []
-    for filename in [
-        "batch_decode_mla_plan.cu",
-        "batch_decode_mla_run.cu",
-        "batch_decode_mla_pybind.cu",
-    ]:
+    for filename in filenames:
         src_path = FLASHINFER_CSRC_DIR / filename
         dest_path = gen_directory / filename
         source_paths.append(dest_path)
@@ -218,7 +324,8 @@ def get_single_prefill_uri(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -228,7 +335,8 @@ def get_single_prefill_uri(
         f"single_prefill_with_kv_cache_dtype_q_{filename_safe_dtype_map[dtype_q]}_"
         f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
         f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
-        f"head_dim_{head_dim}_"
+        f"head_dim_qk_{head_dim_qk}_"
+        f"head_dim_vo_{head_dim_vo}_"
         f"posenc_{pos_encoding_mode}_"
         f"use_swa_{use_sliding_window}_"
         f"use_logits_cap_{use_logits_soft_cap}_"
@@ -242,7 +350,8 @@ def get_batch_prefill_uri(
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
     dtype_idx: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -253,7 +362,8 @@ def get_batch_prefill_uri(
         f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
         f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
         f"dtype_idx_{filename_safe_dtype_map[dtype_idx]}_"
-        f"head_dim_{head_dim}_"
+        f"head_dim_qk_{head_dim_qk}_"
+        f"head_dim_vo_{head_dim_vo}_"
         f"posenc_{pos_encoding_mode}_"
         f"use_swa_{use_sliding_window}_"
         f"use_logits_cap_{use_logits_soft_cap}_"
@@ -265,7 +375,8 @@ def gen_single_decode_module(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -274,7 +385,8 @@ def gen_single_decode_module(
         dtype_q,
         dtype_kv,
         dtype_o,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         pos_encoding_mode,
         use_sliding_window,
         use_logits_soft_cap,
@@ -284,7 +396,8 @@ def gen_single_decode_module(
         dtype_q,
         dtype_kv,
         dtype_o,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         ["maybe_alibi_slopes"],  # additional_tensor_names
         ["float"],  # additional_tensor_dtypes
         [
@@ -293,7 +406,7 @@ def gen_single_decode_module(
             "rope_rcp_scale",
             "rope_rcp_theta",
         ],  # additional_scalar_names
-        ["float", "float", "float", "float"],  # additional_scalar_dtypes
+        ["double", "double", "double", "double"],  # additional_scalar_dtypes
         f"DefaultAttention<false, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>",  # variant_name
         f"#include<flashinfer/attention/variants.cuh>",  # variant_decl
         pos_encoding_mode=pos_encoding_mode,
@@ -307,7 +420,8 @@ def gen_single_prefill_module(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -318,7 +432,8 @@ def gen_single_prefill_module(
         dtype_q,
         dtype_kv,
         dtype_o,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         pos_encoding_mode,
         use_sliding_window,
         use_logits_soft_cap,
@@ -333,14 +448,14 @@ def gen_single_prefill_module(
             "rope_rcp_scale",
             "rope_rcp_theta",
         ]
-        additional_scalar_dtypes = ["float", "float", "float", "float"]
+        additional_scalar_dtypes = ["double", "double", "double", "double"]
         variant_name = f"DefaultAttention<use_custom_mask, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>"
         variant_decl = f"#include<flashinfer/attention/variants.cuh>"
     else:
         additional_tensor_names = []
         additional_tensor_dtypes = []
         additional_scalar_names = ["logits_soft_cap", "sm_scale"]
-        additional_scalar_dtypes = ["float", "float"]
+        additional_scalar_dtypes = ["double", "double"]
         variant_name = f"DefaultAttention<{str(use_logits_soft_cap).lower()}>"
         variant_decl = f"#include<flashinfer/attention/hopper/variants.cuh>"
 
@@ -350,7 +465,8 @@ def gen_single_prefill_module(
         dtype_q,
         dtype_kv,
         dtype_o,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         additional_tensor_names,
         additional_tensor_dtypes,
         additional_scalar_names,
@@ -369,7 +485,8 @@ def gen_batch_decode_module(
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
     dtype_idx: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -379,7 +496,8 @@ def gen_batch_decode_module(
         dtype_kv,
         dtype_o,
         dtype_idx,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         pos_encoding_mode,
         use_sliding_window,
         use_logits_soft_cap,
@@ -390,7 +508,8 @@ def gen_batch_decode_module(
         dtype_kv,
         dtype_o,
         dtype_idx,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         ["maybe_alibi_slopes"],  # additional_tensor_names
         ["float"],  # additional_tensor_dtypes
         [
@@ -399,7 +518,7 @@ def gen_batch_decode_module(
             "rope_rcp_scale",
             "rope_rcp_theta",
         ],  # additional_scalar_names
-        ["float", "float", "float", "float"],  # additional_scalar_dtypes
+        ["double", "double", "double", "double"],  # additional_scalar_dtypes
         f"DefaultAttention<false, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>",  # variant_name
         f"#include<flashinfer/attention/variants.cuh>",  # variant_decl
         pos_encoding_mode=pos_encoding_mode,
@@ -414,7 +533,8 @@ def gen_batch_prefill_module(
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
     dtype_idx: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     pos_encoding_mode: int,
     use_sliding_window: bool,
     use_logits_soft_cap: bool,
@@ -426,7 +546,8 @@ def gen_batch_prefill_module(
         dtype_kv,
         dtype_o,
         dtype_idx,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         pos_encoding_mode,
         use_sliding_window,
         use_logits_soft_cap,
@@ -450,14 +571,14 @@ def gen_batch_prefill_module(
             "rope_rcp_scale",
             "rope_rcp_theta",
         ]
-        additional_scalar_dtypes = ["float", "float", "float", "float"]
+        additional_scalar_dtypes = ["double", "double", "double", "double"]
         variant_name = f"DefaultAttention<use_custom_mask, {str(use_sliding_window).lower()}, {str(use_logits_soft_cap).lower()}, {str(pos_encoding_mode == 2).lower()}>"
         variant_decl = f"#include<flashinfer/attention/variants.cuh>"
     else:
         additional_tensor_names = []
         additional_tensor_dtypes = []
         additional_scalar_names = ["logits_soft_cap", "sm_scale"]
-        additional_scalar_dtypes = ["float", "float"]
+        additional_scalar_dtypes = ["double", "double"]
         variant_name = f"DefaultAttention<{str(use_logits_soft_cap).lower()}>"
         variant_decl = f"#include<flashinfer/attention/hopper/variants.cuh>"
 
@@ -468,7 +589,8 @@ def gen_batch_prefill_module(
         dtype_kv,
         dtype_o,
         dtype_idx,
-        head_dim,
+        head_dim_qk,
+        head_dim_vo,
         additional_tensor_names,
         additional_tensor_dtypes,
         additional_scalar_names,
@@ -487,7 +609,8 @@ def gen_customize_single_decode_module(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     additional_tensor_names: List[str],
     additional_tensor_dtypes: List[str],
     additional_scalar_names: List[str],
@@ -526,7 +649,8 @@ def gen_customize_single_decode_module(
         "dtype_q": dtype_map[dtype_q],
         "dtype_kv": dtype_map[dtype_kv],
         "dtype_o": dtype_map[dtype_o],
-        "head_dim": head_dim,
+        "head_dim_qk": head_dim_qk,
+        "head_dim_vo": head_dim_vo,
         "pos_encoding_mode": pos_encoding_mode_literal[pos_encoding_mode],
         "use_sliding_window": str(use_sliding_window).lower(),
         "use_logits_soft_cap": str(use_logits_soft_cap).lower(),
@@ -570,7 +694,8 @@ def gen_customize_single_prefill_module(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     additional_tensor_names: List[str],
     additional_tensor_dtypes: List[str],
     additional_scalar_names: List[str],
@@ -588,7 +713,8 @@ def gen_customize_single_prefill_module(
         "dtype_q": dtype_map[dtype_q],
         "dtype_kv": dtype_map[dtype_kv],
         "dtype_o": dtype_map[dtype_o],
-        "head_dim": head_dim,
+        "head_dim_qk": head_dim_qk,
+        "head_dim_vo": head_dim_vo,
         "pos_encoding_mode": pos_encoding_mode_literal[pos_encoding_mode],
         "use_sliding_window": str(use_sliding_window).lower(),
         "use_logits_soft_cap": str(use_logits_soft_cap).lower(),
@@ -721,7 +847,8 @@ def gen_customize_batch_decode_module(
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
     idtype: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     additional_tensor_names: List[str],
     additional_tensor_dtypes: List[str],
     additional_scalar_names: List[str],
@@ -752,7 +879,8 @@ def gen_customize_batch_decode_module(
         "dtype_kv": dtype_map[dtype_kv],
         "dtype_o": dtype_map[dtype_o],
         "idtype": dtype_map[idtype],
-        "head_dim": head_dim,
+        "head_dim_qk": head_dim_qk,
+        "head_dim_vo": head_dim_vo,
         "pos_encoding_mode": pos_encoding_mode_literal[pos_encoding_mode],
         "use_sliding_window": str(use_sliding_window).lower(),
         "use_logits_soft_cap": str(use_logits_soft_cap).lower(),
@@ -803,7 +931,8 @@ def gen_customize_batch_prefill_module(
     dtype_kv: torch.dtype,
     dtype_o: torch.dtype,
     idtype: torch.dtype,
-    head_dim: int,
+    head_dim_qk: int,
+    head_dim_vo: int,
     additional_tensor_names: List[str],
     additional_tensor_dtypes: List[str],
     additional_scalar_names: List[str],
@@ -822,7 +951,8 @@ def gen_customize_batch_prefill_module(
         "dtype_kv": dtype_map[dtype_kv],
         "dtype_o": dtype_map[dtype_o],
         "idtype": dtype_map[idtype],
-        "head_dim": head_dim,
+        "head_dim_qk": head_dim_qk,
+        "head_dim_vo": head_dim_vo,
         "pos_encoding_mode": pos_encoding_mode_literal[pos_encoding_mode],
         "use_sliding_window": str(use_sliding_window).lower(),
         "use_logits_soft_cap": str(use_logits_soft_cap).lower(),
