@@ -355,6 +355,7 @@ class BlockSparseAttentionWrapper:
                 q_data_type,
                 indptr.dtype,
                 head_dim,
+                head_dim,
                 PosEncodingMode[pos_encoding_mode].value,
                 False,  # use_sliding_window
                 logits_soft_cap > 0,  # use_logits_soft_cap
@@ -374,6 +375,7 @@ class BlockSparseAttentionWrapper:
                     -1,  # window_left
                     logits_soft_cap,  # logits_soft_cap
                     head_dim,
+                    head_dim,
                     torch.empty(0, dtype=q_data_type),
                     torch.empty(0, dtype=kv_data_type),
                     get_cuda_stream(device),
@@ -397,7 +399,8 @@ class BlockSparseAttentionWrapper:
                 kv_data_type,
                 q_data_type,
                 indptr.dtype,
-                head_dim,
+                head_dim,  # head_dim_qk
+                head_dim,  # head_dim_vo
                 PosEncodingMode[pos_encoding_mode].value,
                 False,  # use_sliding_window
                 logits_soft_cap > 0,  # use_logits_soft_cap
@@ -441,6 +444,7 @@ class BlockSparseAttentionWrapper:
                     self.C,  # page_size
                     False,  # is_cuda_graph_enabled,
                     head_dim,
+                    head_dim,
                     causal,
                     get_cuda_stream(device),
                 )
