@@ -171,7 +171,7 @@ def _get_cache_alibi_slopes_buf(
     key = (f"alibi_slopes_{num_qo_heads}", device)
     buf = _cache_buf.get(key)
     if buf is None:
-        buf = (get_alibi_slopes(num_qo_heads) * log2e).to(device)
+        buf = get_alibi_slopes(num_qo_heads).to(device)
         _cache_buf[key] = buf
     return buf
 
@@ -362,3 +362,24 @@ def determine_attention_backend(
         return "fa3"
     else:
         return "fa2"
+
+
+def _check_shape_dtype_device(
+    x: torch.Tensor,
+    expected_shape: Sequence[int],
+    expected_dtype: torch.dtype,
+    expected_device: torch.device,
+    name: str,
+) -> None:
+    if x.shape != torch.Size(expected_shape):
+        raise ValueError(
+            f"Invalid shape of {name}: expected {expected_shape}, got {x.shape}"
+        )
+    if x.dtype != expected_dtype:
+        raise ValueError(
+            f"Invalid dtype of {name}: expected {expected_dtype}, got {x.dtype}"
+        )
+    if x.device != expected_device:
+        raise ValueError(
+            f"Invalid device of {name}: expected {expected_device}, got {x.device}"
+        )
