@@ -30,7 +30,11 @@ def warmup_jit():
             flashinfer.jit.parallel_load_modules(
                 jit_decode_attention_func_args(
                     [torch.float16],  # q_dtypes
-                    [torch.float16, torch.float8_e4m3fn, torch.float8_e5m2],  # kv_dtypes
+                    [
+                        torch.float16,
+                        torch.float8_e4m3fn,
+                        torch.float8_e5m2,
+                    ],  # kv_dtypes
                     [128, 256],  # head_dims
                     [0, 1, 2],  # pos_encoding_modes
                     [False],  # use_sliding_windows
@@ -38,12 +42,16 @@ def warmup_jit():
                 )
                 + jit_prefill_attention_func_args(
                     [torch.float16],  # q_dtypes
-                    [torch.float16, torch.float8_e4m3fn, torch.float8_e5m2],  # kv_dtypes
+                    [
+                        torch.float16,
+                        torch.float8_e4m3fn,
+                        torch.float8_e5m2,
+                    ],  # kv_dtypes
                     [128, 256],  # head_dims
                     [0, 1, 2],  # pos_encoding_modes
                     [False],  # use_sliding_windows
                     [False, True],  # use_logits_soft_caps
-                    [False],  # allow_fp16_qk_reductions
+                    [False],  # use_fp16_qk_reductions
                 )
             )
         except Exception as e:
@@ -453,7 +461,6 @@ def test_cuda_graph_batch_decode_with_paged_kv_cache(
 
     # compute ground truth and compare
     kv_indptr = kv_indptr_host.to(0)
-    kv_indices = kv_indices_host.to(0)
     kv_last_page_len = kv_last_page_len_host.to(0)
     for i in range(batch_size):
         perm_dims = [0, 2, 1, 3] if kv_layout == "HND" else [0, 1, 2, 3]
