@@ -30,12 +30,16 @@ def warmup_jit():
             flashinfer.jit.parallel_load_modules(
                 jit_prefill_attention_func_args(
                     [torch.float16],  # q_dtypes
-                    [torch.float16, torch.float8_e4m3fn, torch.float8_e5m2],  # kv_dtypes
+                    [
+                        torch.float16,
+                        torch.float8_e4m3fn,
+                        torch.float8_e5m2,
+                    ],  # kv_dtypes
                     [128, 256],  # head_dims
                     [0, 1, 2],  # pos_encoding_modes
                     [False],  # use_sliding_windows
                     [False, True],  # use_logits_soft_caps
-                    [False],  # allow_fp16_qk_reductions
+                    [False],  # use_fp16_qk_reductions
                 )
             )
         except Exception as e:
@@ -543,7 +547,7 @@ def test_batch_prefill_with_paged_kv_cache_custom_mask(
         num_kv_heads,
         head_dim,
         page_size,
-        custom_mask,
+        custom_mask=custom_mask,
         pos_encoding_mode=pos_encoding_mode,
         logits_soft_cap=logits_soft_cap,
     )
