@@ -18,11 +18,12 @@ from flashinfer.utils import MaskMode
 def test_single_decode_mask():
     torch.manual_seed(42)
     variant_decl = r"""
-struct SingleDecodeWithCustomMask {
+struct SingleDecodeWithCustomMask : AttentionVariantBase {
   static constexpr bool use_softmax = true;
 
   uint8_t* custom_mask_ptr;
   uint32_t window_left, qo_len, kv_len;
+  float sm_scale_log2;
 
   // Create closure
   template <typename Params>
@@ -32,27 +33,13 @@ struct SingleDecodeWithCustomMask {
     qo_len = 1;
     kv_len = params.get_kv_len(batch_idx);
     window_left = kv_len;
+    sm_scale_log2 = params.sm_scale * math::log2e;
   }
 
-  template <typename Params, typename T>
-  __device__ __forceinline__ T QueryTransform(const Params& params, T q) {
-    return float(q) * params.sm_scale * math::log2e;
-  }
-
-  template <typename Params, typename T>
-  __device__ __forceinline__ T LogitsTransform(const Params& params, T logits, uint32_t batch_idx,
-                                               uint32_t qo_idx, uint32_t kv_idx,
-                                               uint32_t qo_head_idx, uint32_t kv_head_idx) {
-    return logits;
-  }
-
-  template <typename Params>
-  __device__ __forceinline__ bool LogitsMask(const Params& params, uint32_t batch_idx,
-                                             uint32_t qo_idx, uint32_t kv_idx, uint32_t qo_head_idx,
-                                             uint32_t kv_head_idx) {
+  REGISTER_LOGITS_MASK(params, batch_idx, qo_idx, kv_idx, qo_head_idx, kv_head_idx, {
     const uint32_t offset = kv_idx;
     return ((custom_mask_ptr[offset / 8] >> (offset % 8)) & 1);
-  }
+  })
 };
 """
     jit_module = gen_customize_single_decode_module(
@@ -60,7 +47,8 @@ struct SingleDecodeWithCustomMask {
         torch.float16,  # dtype_q
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
-        128,  # head_dim
+        128,  # head_dim_qk
+        128,  # head_dim_vo
         ["custom_mask"],  # additional_tensor_names
         ["uint8_t"],  # additional_tensor_dtypes
         ["sm_scale"],  # # additional_scalar_names
@@ -88,11 +76,12 @@ struct SingleDecodeWithCustomMask {
 
 
 flash_sigmoid_sm80_decl = r"""
-struct FlashSigmoid {
+struct FlashSigmoid : AttentionVariantBase {
   static constexpr bool use_softmax = false;
 
   uint32_t window_left, qo_len, kv_len;
-  float sigmoid_bias_log2e;
+  float sigmoid_scale_log2;
+  float sigmoid_bias_log2;
 
   // Create closure
   template <typename Params>
@@ -101,35 +90,18 @@ struct FlashSigmoid {
     qo_len = params.get_qo_len(batch_idx);
     kv_len = params.get_kv_len(batch_idx);
     window_left = kv_len;
-    sigmoid_bias_log2e = params.sigmoid_bias * math::log2e;
+    sigmoid_bias_log2 = params.sigmoid_bias * math::log2e;
+    sigmoid_scale_log2 = params.logits_scale * math::log2e;
   }
 
-  template <typename Params, typename T>
-  __device__ __forceinline__ T QueryTransform(const Params& params, T q) {
-    return float(q) * params.logits_scale * math::log2e;
-  }
-
-  template <typename Params, typename T>
-  __device__ __forceinline__ T LogitsTransform(const Params& params, T logits, uint32_t batch_idx,
-                                               uint32_t qo_idx, uint32_t kv_idx,
-                                               uint32_t qo_head_idx, uint32_t kv_head_idx) {
-    return math::ptx_rcp(1.f + math::ptx_exp2(-float(logits + sigmoid_bias_log2e)));
-  }
-
-  template <typename Params>
-  __device__ __forceinline__ bool LogitsMask(const Params& params, uint32_t batch_idx,
-                                             uint32_t qo_idx, uint32_t kv_idx, uint32_t qo_head_idx,
-                                             uint32_t kv_head_idx) {
-    return true;
-  }
+  REGISTER_LOGITS_TRANSFORM(params, logits, batch_idx, qo_idx, kv_idx, qo_head_idx, kv_head_idx, {
+    return math::ptx_rcp(1.f + math::ptx_exp2(-float(logits * sigmoid_scale_log2 + sigmoid_bias_log2)));
+  });
 };
 """
 
 flash_sigmoid_sm90_decl = r"""
-struct FlashSigmoid {
-  template <int NUM_ROWS_PER_THREAD>
-  using Updater = DefaultUpdater<NUM_ROWS_PER_THREAD>;
-
+struct FlashSigmoid : AttentionVariantBase {
   float logits_scale_log2, sigmoid_bias_log2e;
   // Init
   template <typename MainloopParams, typename BlockCoord>
@@ -138,13 +110,15 @@ struct FlashSigmoid {
     sigmoid_bias_log2e = params.additional_params.sigmoid_bias * math::log2e;
   }
 
-  template <typename MainloopParams, typename T>
-  __device__ __forceinline__ T LogitsTransform(const MainloopParams& params, T logits,
-                                               int batch_idx,
-                                               int qo_idx, int kv_idx,
-                                               int qo_head_idx, int kv_head_idx) {
-    return math::ptx_rcp(1.f + math::ptx_exp2(-float(logits * logits_scale_log2 + sigmoid_bias_log2e)));
+
+  template <int NUM_ROWS_PER_THREAD>
+  __device__ auto GetAttentionUpdater() {
+    return DefaultUpdater<NUM_ROWS_PER_THREAD>();
   }
+
+  REGISTER_LOGITS_TRANSFORM(params, logits, batch_idx, qo_idx, kv_idx, qo_head_idx, kv_head_idx, {
+    return math::ptx_rcp(1.f + math::ptx_exp2(-float(logits * logits_scale_log2 + sigmoid_bias_log2e)));
+  });
 };
 """
 
@@ -158,7 +132,8 @@ def test_flash_sigmoid():
         torch.float16,  # dtype_q
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
-        128,  # hidden_dim
+        128,  # head_dim_qk
+        128,  # head_dim_vo
         [],  # additional_tensor_names
         [],  # additional_tensor_dtypes
         ["logits_scale", "sigmoid_bias"],  # additional_scalar_names
@@ -186,10 +161,11 @@ def test_flash_sigmoid():
 def test_dump_logits():
     torch.manual_seed(42)
     variant_decl = r"""
-struct DumpLogits {
+struct DumpLogits : AttentionVariantBase {
   static constexpr bool use_softmax = true;
 
   uint32_t window_left, qo_len, kv_len;
+  float sm_scale_log2;
 
   // Create closure
   template <typename Params>
@@ -198,29 +174,15 @@ struct DumpLogits {
     qo_len = params.get_qo_len(batch_idx);
     kv_len = params.get_kv_len(batch_idx);
     window_left = kv_len;
+    sm_scale_log2 = params.sm_scale * math::log2e;
   }
 
-  template <typename Params, typename T>
-  __device__ __forceinline__ T QueryTransform(const Params& params, T q) {
-    return float(q) * params.sm_scale * math::log2e;
-  }
-
-  template <typename Params, typename T>
-  __device__ __forceinline__ T LogitsTransform(const Params& params, T logits, uint32_t batch_idx,
-                                               uint32_t qo_idx, uint32_t kv_idx,
-                                               uint32_t qo_head_idx, uint32_t kv_head_idx) {
+  REGISTER_LOGITS_TRANSFORM(params, logits, batch_idx, qo_idx, kv_idx, qo_head_idx, kv_head_idx, {
     if (qo_idx < qo_len && kv_idx < kv_len) {
-      params.output_logits[qo_head_idx * (qo_len * kv_len) + qo_idx * kv_len + kv_idx] = logits * math::loge2;
+      params.output_logits[qo_head_idx * (qo_len * kv_len) + qo_idx * kv_len + kv_idx] = logits * params.sm_scale;
     }
     return logits;
-  }
-
-  template <typename Params>
-  __device__ __forceinline__ bool LogitsMask(const Params& params, uint32_t batch_idx,
-                                             uint32_t qo_idx, uint32_t kv_idx, uint32_t qo_head_idx,
-                                             uint32_t kv_head_idx) {
-    return true;
-  }
+  });
 };
 """
     jit_module = gen_customize_single_prefill_module(
@@ -229,7 +191,8 @@ struct DumpLogits {
         torch.float16,  # dtype_q
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
-        128,  # hidden_dim
+        128,  # head_dim_qk
+        128,  # head_dim_vo
         ["output_logits"],  # additional_tensor_names
         ["float"],  # additional_tensor_dtypes
         ["sm_scale"],  # additional_scalar_names
@@ -263,7 +226,8 @@ def test_batch_decode_flash_sigmoid(use_tensor_cores):
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
         torch.int32,  # idtype
-        128,  # hidden_dim
+        128,  # hidden_dim_qk
+        128,  # hidden_dim_vo
         [],  # additional_tensor_names
         [],  # additional_tensor_dtypes
         ["logits_scale", "sigmoid_bias"],  # additional_scalar_names
@@ -369,7 +333,8 @@ def test_batch_prefill_flash_sigmoid():
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
         torch.int32,  # idtype
-        128,  # hidden_dim
+        128,  # hidden_dim_qk
+        128,  # hidden_dim_vo
         [],  # additional_tensor_names
         [],  # additional_tensor_dtypes
         ["logits_scale", "sigmoid_bias"],  # additional_scalar_names
@@ -487,7 +452,8 @@ def test_batch_prefill_sm90_flash_sigmoid():
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
         torch.int32,  # idtype
-        128,  # hidden_dim
+        128,  # hidden_dim_qk
+        128,  # hidden_dim_vo
         [],  # additional_tensor_names
         [],  # additional_tensor_dtypes
         ["logits_scale", "sigmoid_bias"],  # additional_scalar_names
@@ -598,10 +564,11 @@ def test_batch_prefill_sm90_flash_sigmoid():
 def test_debug_print_logits():
     torch.manual_seed(42)
     variant_decl = r"""
-struct DebugPrintLogits {
+struct DebugPrintLogits : AttentionVariantBase {
   static constexpr bool use_softmax = true;
 
   uint32_t window_left, qo_len, kv_len;
+  float sm_scale_log2;
 
   // Create closure
   template <typename Params>
@@ -610,30 +577,16 @@ struct DebugPrintLogits {
     qo_len = params.get_qo_len(batch_idx);
     kv_len = params.get_kv_len(batch_idx);
     window_left = kv_len;
+    sm_scale_log2 = params.sm_scale * math::log2e;
   }
 
-  template <typename Params, typename T>
-  __device__ __forceinline__ T QueryTransform(const Params& params, T q) {
-    return float(q) * params.sm_scale * math::log2e;
-  }
-
-  template <typename Params, typename T>
-  __device__ __forceinline__ T LogitsTransform(const Params& params, T logits, uint32_t batch_idx,
-                                               uint32_t qo_idx, uint32_t kv_idx,
-                                               uint32_t qo_head_idx, uint32_t kv_head_idx) {
+  REGISTER_LOGITS_TRANSFORM(params, logits, batch_idx, qo_idx, kv_idx, qo_head_idx, kv_head_idx, {
     if (logits >= 5) {
       printf("Large logits at qo_idx=%d, kv_idx=%d, qo_head_idx=%d, kv_head_idx=%d: %.3f\n",
-             qo_idx, kv_idx, qo_head_idx, kv_head_idx, float(logits));
+              qo_idx, kv_idx, qo_head_idx, kv_head_idx, float(logits));
     }
     return logits;
-  }
-
-  template <typename Params>
-  __device__ __forceinline__ bool LogitsMask(const Params& params, uint32_t batch_idx,
-                                             uint32_t qo_idx, uint32_t kv_idx, uint32_t qo_head_idx,
-                                             uint32_t kv_head_idx) {
-    return true;
-  }
+  });
 };
 """
     jit_module = gen_customize_single_prefill_module(
@@ -642,7 +595,8 @@ struct DebugPrintLogits {
         torch.float16,  # dtype_q
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
-        128,  # hidden_dim
+        128,  # hidden_dim_qk
+        128,  # hidden_dim_vo
         [],  # additional_tensor_names
         [],  # additional_tensor_dtypes
         ["sm_scale"],  # additional_scalar_names
@@ -667,11 +621,7 @@ struct DebugPrintLogits {
 def test_sm90_debug_print_logits():
     torch.manual_seed(42)
     variant_decl = r"""
-struct DebugPrintLogits {
-
-  template <int NUM_ROWS_PER_THREAD>
-  using Updater = OnlineSoftmaxWithoutScale<NUM_ROWS_PER_THREAD>;
-
+struct DebugPrintLogits : AttentionVariantBase {
   float sm_scale_log2;
   int qo_len, kv_len;
 
@@ -686,27 +636,30 @@ struct DebugPrintLogits {
     kv_len = kv_len_;
   }
 
-  template <typename MainloopParams, typename T>
-  __device__ __forceinline__ T LogitsTransform(const MainloopParams& params, T logits,
-                                               int batch_idx,
-                                               int qo_idx, int kv_idx,
-                                               int qo_head_idx, int kv_head_idx) {
+
+  template <int NUM_ROWS_PER_THREAD>
+  __device__ auto GetAttentionUpdater() {
+    return OnlineSoftmax<NUM_ROWS_PER_THREAD, /*WITH_SCALE*/false>(sm_scale_log2);
+  }
+
+
+  REGISTER_LOGITS_TRANSFORM(params, logits, batch_idx, qo_idx, kv_idx, qo_head_idx, kv_head_idx, {
     if (qo_idx < qo_len && kv_idx < kv_len) {
-      printf(
-          "---> LOGITS DEBUG: "
-          "qo_idx=%-5d "
-          "kv_idx=%-5d "
-          "sm_scale_log2=%-12.5f "
-          "logits=%-12.5f "
-          "\n",
-          qo_idx,
-          kv_idx,
-          sm_scale_log2,
-          static_cast<float>(logits));
+        printf(
+            "---> LOGITS DEBUG: "
+            "qo_idx=%-5d "
+            "kv_idx=%-5d "
+            "sm_scale_log2=%-12.5f "
+            "logits=%-12.5f "
+            "\n",
+            qo_idx,
+            kv_idx,
+            sm_scale_log2,
+            static_cast<float>(logits));
     }
     logits *= sm_scale_log2;
     return logits;
-  }
+  })
 };
 """
     jit_module = gen_customize_single_prefill_module(
@@ -715,7 +668,8 @@ struct DebugPrintLogits {
         torch.float16,  # dtype_q
         torch.float16,  # dtype_kv
         torch.float16,  # dtype_o
-        128,  # hidden_dim
+        128,  # hidden_dim_qk
+        128,  # hidden_dim_vo
         [],  # additional_tensor_names
         [],  # additional_tensor_dtypes
         ["sm_scale"],  # additional_scalar_names
@@ -738,12 +692,12 @@ struct DebugPrintLogits {
 
 
 if __name__ == "__main__":
-    # test_single_decode_mask()
-    # test_flash_sigmoid()
-    # test_dump_logits()
-    # test_debug_print_logits()
-    # test_sm90_debug_print_logits()
-    # test_batch_decode_flash_sigmoid(False)
-    # test_batch_decode_flash_sigmoid(True)
-    # test_batch_prefill_flash_sigmoid()
+    test_single_decode_mask()
+    test_flash_sigmoid()
+    test_dump_logits()
+    test_debug_print_logits()
+    test_sm90_debug_print_logits()
+    test_batch_decode_flash_sigmoid(False)
+    test_batch_decode_flash_sigmoid(True)
+    test_batch_prefill_flash_sigmoid()
     test_batch_prefill_sm90_flash_sigmoid()
