@@ -316,6 +316,16 @@ class DeepseekV2AttentionMatAbsorbDecode(nn.Module):
                 raise ValueError(
                     "For simplicity, kv_len should be multiple of page_size."
                 )
+            freqs_cis = precompute_freqs_cis(
+                self.qk_rope_head_dim, kv_len, self.rope_theta, use_scaled=False
+            ).to(k_pe_cache.device)
+            q_pe, k_pe_cache = apply_rotary_emb(
+                q_pe.unsqueeze(1).repeat(1, kv_len, 1, 1),
+                k_pe_cache.unsqueeze(2),
+                freqs_cis,
+            )
+            q_pe = q_pe[:, -1:, :, :].squeeze(1).contiguous()
+            k_pe_cache = k_pe_cache.squeeze(2)
             num_pages_per_seq = kv_len // page_size
             total_num_pages = num_pages_per_seq * bsz
 
@@ -335,7 +345,14 @@ class DeepseekV2AttentionMatAbsorbDecode(nn.Module):
             workspace_buffer = torch.empty(32 * 1024 * 1024, dtype=torch.int8).to(
                 dev_id
             )
-            wrapper = flashinfer.BatchDecodeMlaWithPagedKVCacheWrapper(workspace_buffer)
+            wrapper = flashinfer.BatchDecodeMlaWithPagedKVCacheWrapper(
+                workspace_buffer, 
+                use_cuda_graph=True,
+                paged_kv_indptr_buffer=kv_indptr,
+                paged_kv_indices_buffer=kv_indices,
+                paged_kv_last_page_len_buffer=kv_last_page_len,
+            )
+            
             wrapper.plan(
                 kv_indptr,
                 kv_indices,
@@ -348,8 +365,12 @@ class DeepseekV2AttentionMatAbsorbDecode(nn.Module):
                 data_type=q_kv_dtype,
                 q_data_type=q_kv_dtype,
             )
-            attn_output = wrapper.run(q_nope, q_pe, paged_ckv_cache, paged_kpe_cache)
-
+            
+            g = torch.cuda.CUDAGraph()
+            with torch.cuda.graph(g):
+                attn_output = wrapper.run(q_nope, q_pe, paged_ckv_cache, paged_kpe_cache)
+            g.replay()
+        
         # output ~ [bsz, 5120]
         output = torch.matmul(
             attn_output.to(self.W_UV_O.dtype).reshape(
@@ -363,7 +384,7 @@ class DeepseekV2AttentionMatAbsorbDecode(nn.Module):
 
 if __name__ == "__main__":
 
-    dev_id = 1
+    dev_id = 0
 
     torch.manual_seed(666)
     torch.set_grad_enabled(False)
